#!/bin/sh

set -e

errexit ()
{
	printf "\033[31mERROR:\033[0m $1\n"
	exit 1
}

warn ()
{
	printf "\033[31mWARNING:\033[0m $1\n"
}

findcmd ()
{
	ret=`whereis -sb $1 | cut -d " " -f2`
	if [ "$ret" = "$1: " ]; then
		printf %s\\n ''
	fi

	printf %s\\n "$ret"
}

# Uses 'file' to figure out whether a binary is 32 or 64 bits.
# Usage: archtype <binary>, returns: '32' or '64'
archtype ()
{
	printf %s\\n `file "$1" | sed -n 's/.*\([0-9][0-9]\)-bit.*/\1/p'`
}

# Returns absolute path to a file.
getabspath ()
{
	printf %s\\n `readlink -f "$1"`
}

# Returns absolute location to a file or directory.
getabsdir ()
{
	printf %s\\n `dirname \`getabspath "$1")\``
}

# Creates a symlink in the PRS installation directory to a suitable
# version of the ODBC library which it finds using 'ldconfig -p'.
# Arguments: $1 Location to place the link in. $2 Name of the binary.
odbcsymlink ()
{
	# Skip symlinking if we do not have PRS install dir write access.
	if [ ! -w "$1" ]; then
		warn "No write access to directory ($1), skipping symlinking ODBC library."
		return 0
	fi

	# Get abs. path to the ldconfig command, might not be in default path
	# or even in the standard location (Arch Linux for example).
	ldconfig=`findcmd ldconfig`
	if [ -z "$ldconfig" ]; then
		errexit "Unable to find ldconfig command. Aborting."
	else
		printf %s\\n "Using $ldconfig to locate libraries."
	fi

	# Figure out which architecture the binary is compiled for.
	if [ ! -e "$1/$2" ]; then
		errexit "Unable to find binary ($1/$2). Aborting."
	fi

	prspath=`getabspath "$1/$2"`
	arch=`archtype "$prspath"`
	printf %s\\n "Architecture of $prspath is: $arch"

	# Use ldd to figure out what odbc library PRS wants.
	odbclib=`ldd $prspath | grep odbc | cut -d ' ' -f 1 | tr -d '[[:space:]]'`

	# This is our final link path: <working-dir>/<odbclibname>
	linkloc="$1/$odbclib"

	printf %s\\n "ODBC library linked into binary: $odbclib"

	# Use ldconfig to list installed libraries, extract the
	# columns with architecture and abs. path.
	$ldconfig -p | sed -e "s/'/\\'/g" | grep '[[:space:]]libodbc\.so\.' | cut -d ' ' -f2,4 | while read line ; do

		itemarch=`printf %s "$line" | cut -d ' ' -f1`
		itemname=`printf %s "$line" | cut -d ' ' -f2`
		if [ "$arch" = '64' ]; then
			printf %s\\n "$itemarch" | grep -iq 'x86-64'
		else
			printf %s\\n "$itemarch" | grep -ivq 'x86-64'
		fi

		# If we found a suitable library we proceed.
		if [ $? -eq 0 ]; then

			# If a symlink already exists and points to a valid target
			# we abort here, since nothing needs to be done.
			if [ -e "$linkloc" ]; then
				printf %s\\n "Symlink ($linkloc) exists, not recreating. Manually remove symlink to force recreation."
				return 0
			fi
			printf %s\\n "Creating symbolic link $linkloc pointing to $itemname"
			ln -s "$itemname" "$linkloc"

			# Check to see if we found a matching ODBC library to link to.
			if [ ! -e "$linkloc" ]; then
				printf %s\\n "Unable to find a matching ODBC library."
				printf %s\\n "Have you installed the unixodbc package?"
				exit 1
			fi

			exit 0
		fi
	done
}
